"""
定时任务管理和执行。

本模块负责定义、调度和执行所有后台定时任务，
例如每日自动登录/登出、QMT重启以及连接状态检查等。

设计说明：
- 使用独立的schedule.Scheduler()实例，避免与RealTraderManager调度器冲突
- 主调度器负责：登录、登出、QMT重启等系统级任务
- RealTraderManager调度器负责：订单管理、数据保存等交易级任务
- 两个调度器完全独立，互不干扰

注意事项：
- RealTraderManager在构造函数中会自动启动自己的调度器
- 登录和登出操作会自动管理RealTraderManager的生命周期
- 避免重复调用start_scheduler()或stop_scheduler()
"""
import schedule
import threading
import time
import subprocess
import os

from config import Config
from logger_config import get_logger
from core.base_trader import BaseTrader
from core.app_state import get_real_trader_manager, login, logout, is_real_mode, _trader_instance, logger as app_logger # 使用 app_state中的logger

# 如果需要独立的 logger，可以取消下面这行的注释
# logger = get_logger("scheduler") 
logger = app_logger # 复用 app_state 的 logger

# 创建主调度器实例（独立于RealTraderManager的调度器）
_main_scheduler = schedule.Scheduler()
_scheduler_thread = None
_stop_event = threading.Event()

def _run_scheduler() -> None:
    """主调度器执行线程的主循环。"""
    logger.info("主调度器线程开始运行")
    while not _stop_event.is_set():
        try:
            _main_scheduler.run_pending()
            # 使用wait代替sleep，这样可以立即响应停止事件
            _stop_event.wait(timeout=1.0)
        except Exception as e:
            logger.error(f"主调度器执行错误: {str(e)}")
            # 发生错误时也要检查停止事件
            if not _stop_event.is_set():
                _stop_event.wait(timeout=10.0)
    
    logger.info("主调度器线程收到停止信号，正在退出")

def _check_reconnect() -> None:
    """定期检查交易实例是否需要重连（仅实盘模式）。"""
    while True:
        try:
            # 直接使用 app_state 中的 _trader_instance
            if is_real_mode() and _trader_instance is not None and hasattr(_trader_instance, 'check_reconnect'):
                _trader_instance.check_reconnect()
            time.sleep(60)  # 每分钟检查一次
        except Exception as e:
            logger.error(f"重连检查错误: {str(e)}")

def login_on_trading_day() -> None:
    """仅在交易日执行登录操作。"""
    if BaseTrader.is_trading_date():
        logger.info("今天是交易日，执行计划的登录操作")
        login() # 调用 app_state.login
        # RealTraderManager在构造函数中已经自动启动调度器，无需重复调用
    else:
        logger.info("今天不是交易日，跳过计划的登录操作")

def logout_on_trading_day() -> None:
    """仅在交易日执行登出操作。"""
    if BaseTrader.is_trading_date():
        logger.info("今天是交易日，执行计划的登出操作")
        logout() # 调用 app_state.logout (会自动停止RealTraderManager调度器)
    else:
        logger.info("今天不是交易日，跳过计划的登出操作")

def restart_qmt() -> None:
    """重启QMT客户端软件。"""
    try:
        if not os.path.exists(Config.XT_LAUNCHER):
            logger.error(f"QMT启动路径不存在: {Config.XT_LAUNCHER}")
            return

        for proc_name in [Config.XT_PROCESS1, Config.XT_PROCESS2]:
            try:
                kill_cmd = f'taskkill /F /IM {proc_name}'
                result = subprocess.run(kill_cmd, shell=True, capture_output=True, text=True, check=False)
                if result.returncode == 0:
                    logger.info(f"已关闭进程: {proc_name}")
                else:
                    # 进程不存在通常返回码 128 或 1
                    if result.returncode not in [1, 128]: 
                        logger.warning(f"关闭进程 {proc_name} 时返回码: {result.returncode}, 输出: {result.stdout}, 错误: {result.stderr}")
            except Exception as e:
                logger.error(f"关闭进程 {proc_name} 时发生异常: {str(e)}")

        time.sleep(3)
        subprocess.Popen(Config.XT_LAUNCHER)
        time.sleep(10)
        logger.info(f"已尝试启动QMT软件: {Config.XT_LAUNCHER}")
    except Exception as e:
        logger.error(f"重启QMT软件时发生错误: {str(e)}")

def restart_qmt_on_trading_day() -> None:
    """仅在交易日执行QMT软件重启操作。"""
    if BaseTrader.is_trading_date():
        logger.info("今天是交易日，执行计划的QMT软件重启操作")
        restart_qmt()
    else:
        logger.info("今天不是交易日，跳过计划的QMT软件重启")

def setup_scheduler() -> threading.Thread:
    """
    设置所有定时任务并启动调度线程。
    
    使用独立的调度器实例，避免与RealTraderManager调度器冲突。

    Returns:
        threading.Thread: 启动的调度器线程实例。
    """
    global _scheduler_thread
    
    # 清理之前的任务（防止重复启动时任务累积）
    _main_scheduler.clear()
    
    # 添加定时任务到独立调度器
    _main_scheduler.every().day.at(Config.MARKET_OPEN_TIME).do(login_on_trading_day)
    _main_scheduler.every().day.at(Config.MARKET_CLOSE_TIME).do(logout_on_trading_day)
    _main_scheduler.every().day.at(Config.XT_RESTART_TIME).do(restart_qmt_on_trading_day)
    
    # 重置停止事件
    _stop_event.clear()
    
    _scheduler_thread = threading.Thread(target=_run_scheduler, name="MainSchedulerThread", daemon=True)
    _scheduler_thread.start()
    logger.info("主调度器已启动")
    
    if is_real_mode():
        reconnect_thread = threading.Thread(target=_check_reconnect, daemon=True)
        reconnect_thread.start()
        logger.info("重连检查线程已启动")
    
    return _scheduler_thread


def stop_scheduler() -> None:
    """
    停止主调度器线程。
    
    正确的停止流程：
    1. 设置停止事件通知线程退出
    2. 等待线程自然结束
    3. 清理线程引用和调度器任务
    """
    global _scheduler_thread
    
    if _scheduler_thread and _scheduler_thread.is_alive():
        logger.info("正在停止主调度器...")
        # 设置停止事件
        _stop_event.set()
        
        # 等待线程结束，设置超时避免无限等待
        try:
            _scheduler_thread.join(timeout=5.0)
            if _scheduler_thread.is_alive():
                logger.warning("主调度器线程未能在超时时间内停止")
            else:
                logger.info("主调度器已成功停止")
        except Exception as e:
            logger.error(f"等待主调度器线程结束时发生异常: {e}")
        
        _scheduler_thread = None
    else:
        logger.info("主调度器未运行或已停止")
        
    # 清理调度器中的所有任务
    _main_scheduler.clear()
    logger.info("已清理主调度器中的所有任务") 